/***************** Part of full PrhoLog with proximity relations ***************

  Prholog-full-prox is a  package for strategic rule-based programming
  with individual, function, sequence, and context variables and
  membership constraints over them. It extends Prholog-full by adding
  a strategy for proximity relations.

  prholog.pl, written by Temur Kutsia and Besik Dundua.
  The main file of the full version of Prholog with proximity relations.
  Last edit: September 24, 2021.
 
  Tested for SWI-Prolog (8.2.1 and later).

  (c) RISC, Johannes Kepler University Linz, Austria

********************************************************************************/

version_year_month_day("1.0.0", 2021, "September", 23).

%-------------------------------------------------------------------------------

:- dynamic(numb_rholog_internal/1).
:- dynamic(error_exit_rholog_internal/0).
:- dynamic(to_abolish_rholog_internal/2).

:- op(1200, xfx, :=).
:- op(130, xfx, ::).
:- op(125, xfx, where).
:- op(120, xfx, in).
:- op(120, xfx, ==>).
:- op(120, xfx, =\=>).
:- op(100, xfy, --->).

:- include('library.pl').

:- multifile user:message_hook/3.

%  Find out the current prholog version.

prholog_version :-
    version_year_month_day(V, Y, M, D),
    format('This is Prholog-full with proximity relations, version ~w, ~w ~d, ~d.~n', [V, M, D, Y]).


              /*************************************
              *                                    *
              *      Loading Prholog files         *
              *                                    *
              *************************************/

load(Rholog_absolute_file_name) :-
    !,
    load(Rholog_absolute_file_name, [delete_prolog_file_at_halt(true)]).
load(Rholog_absolute_file_name, Option) :-
    set_prolog_flag(prompt_alternatives_on, groundness),
    file_directory_name(Rholog_absolute_file_name, Directory),
    file_base_name(Rholog_absolute_file_name, Rholog_relative_file_name),
    catch(rholog_file_name_check(Rholog_relative_file_name, Base),
        E, (print_message(user_error, E),!,fail)),
    file_name_extension(Base, pl, Prolog_relative_file_name),
    absolute_file_name(Prolog_relative_file_name, [relative_to(Directory)],
        Prolog_absolute_file_name),
    open(Rholog_absolute_file_name, read, Rholog_file_stream, 
        [close_on_abort(true)]),
    set_input(Rholog_file_stream),
    open(Prolog_absolute_file_name, write, Prolog_file_stream, 
        [close_on_abort(true)]),
    set_output(Prolog_file_stream),
    writef(':-style_check(-singleton).\n'),
    source_file(Rholog_source_absolute_file_name),
    file_base_name(Rholog_source_absolute_file_name, 'prholog.pl'),
    file_directory_name(Rholog_source_absolute_file_name, 
        Rholog_source_Directory),
    absolute_file_name('library.pl', [relative_to(Rholog_source_Directory)],
        Library_absolute_file_name),
    writef(':-ensure_loaded(%q).\n\n',[Library_absolute_file_name]),
    repeat,
        catch((read_rholog_clause(Rholog_clause),
                check_rholog_clause_syntax(Rholog_clause),
                compile_rholog_clause(Rholog_clause),
                nl),
            E, 
            (print_message(user_error, E), assert(error_exit_rholog_internal))
        ),
    exit_loop(Exit_code),
    !,
    close(Prolog_file_stream),
    close(Rholog_file_stream),
    behaviour_depending_on_loop_exit_code(Exit_code, Prolog_absolute_file_name, 
        Option).

read_rholog_clause(Rholog_clause) :-
    (    read(Rholog_clause),
         !
    ;    throw(error(syntax_error(reading_error),_))
    ).

exit_loop(1) :-
    error_exit_rholog_internal,
    !,
    retract(error_exit_rholog_internal).
exit_loop(0) :-
    at_end_of_stream.

behaviour_depending_on_loop_exit_code(0, Prolog_absolute_file_name, Option) :-
    !,
    consult(Prolog_absolute_file_name),
    at_halt(delete_prolog_file_or_not(Prolog_absolute_file_name,Option)).
behaviour_depending_on_loop_exit_code(1, Prolog_absolute_file_name, _) :-
    (to_abolish_rholog_internal(_, _) ->
         repeat,
             to_abolish_rholog_internal(Pred, Arity),
             abolish(Pred, Arity),
             retract(to_abolish_rholog_internal(Pred, Arity)),
         not(to_abolish_rholog_internal(_, _))
    ),
    delete_file(Prolog_absolute_file_name).

%-------------------------------------------------------------------------------
%
%      rholog_file_name_check(Rholog_relative_file_name, Base).
%      Checks whether Rholog_relative_file_name has an extension 'rho'
%      and if yes, returns the file name base in Base. Otherwise fails.
%
%-------------------------------------------------------------------------------

rholog_file_name_check(Rholog_relative_file_name, Base):-
     file_name_extension(Base, rho, Rholog_relative_file_name),
     !.
rholog_file_name_check(Rholog_relative_file_name, _):-
     throw(error(syntax_error([file_name, Rholog_relative_file_name]),_)).
     
%-------------------------------------------------------------------------------
%
%      delete_prolog_file_or_not(Prolog_absolute_file_name, Options).
%      Deletes the file with the name Prolog_absolute_file_name if
%      Options contain delete_prolog_file_at_halt(true). Keeps the prolog
%      file otherwise.
%
%-------------------------------------------------------------------------------

delete_prolog_file_or_not(Prolog_absolute_file_name, Options):-
    member(delete_prolog_file_at_halt(true), Options),
    exists_file(Prolog_absolute_file_name),
    !,
    delete_file(Prolog_absolute_file_name).
delete_prolog_file_or_not(_, _).

:- include('parse.pl').
:- include('compile.pl').
:- include('solve-prox.pl').


